@@ -147,7 +147,8 @@ class KeyData:
         paths = []
         for chunk in self._data_chunks:
             if chunk.dataset.is_virtual:
-                for vspace, filename, _, _ in chunk.dataset.virtual_sources():
+                mappings = chunk.dataset.virtual_sources()
+                for vspace, filename, _, _ in mappings:
                     if filename in paths:
                         continue  # Already got it
 
@@ -159,9 +160,20 @@ class KeyData:
                     ck = chunk.slice
                     if (map_stop > ck.start) and (map_start < ck.stop):
                         paths.append(filename)
+
+                # Include 1 source file even if no trains are selected
+                if (not paths) and mappings:
+                    paths.append(mappings[0].file_name)
             else:
                 paths.append(chunk.file.filename)
 
+        # Fallback for virtual overview files where no data was recorded for
+        # this source, so there's no mapping to point to.
+        if not paths:
+            source_grp = self.files[0].file[f"{self.section}/{self.source}"]
+            if 'source_files' in source_grp.attrs:
+                paths.append(source_grp.attrs['source_files'][0])
+
         from pathlib import Path
         return [Path(p) for p in paths]
 
@@ -435,12 +447,16 @@ class KeyData:
 
         # Train ID index
         coords = {'trainId': self.train_id_coordinates()}
+        # xarray attributes
+        attrs = {}
+        if (units := self.units):
+            attrs['units'] = units
 
         if ndarr.dtype.names is not None:
             # Structured dtype.
             return xarray.Dataset(
                 {field: (dims, ndarr[field]) for field in ndarr.dtype.names},
-                coords=coords)
+                coords=coords, attrs=attrs)
         else:
             if name is None:
                 name = f'{self.source}.{self.key}'
@@ -450,7 +466,7 @@ class KeyData:
 
             # Primitive dtype.
             return xarray.DataArray(
-                ndarr, dims=dims, coords=coords, name=name)
+                ndarr, dims=dims, coords=coords, name=name, attrs=attrs)
 
     def series(self):
         """Load this data as a pandas Series. Only for 1D data.
